/*
 * Mainboard and BIOS identification
 *
 *   (c) 2002 Bernhard "Bero" Rosenkraenzer <bero@arklinux.org>
 *   (c) 2003, Sergei Haller.
 *
 *   $Id: bios.c,v 1.2 2003/03/18 13:44:01 bistr-o-math Exp $
 *
 * Released under the terms of the GPL version 2 or if, and only if,
 * the GPL version 2 is ruled invalid in a court of law, any later
 * version of the GPL.
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <ctype.h>
#include <syslog.h>

#include "bios.h"
#include "nvram-wakeup.h"

struct header
{
     __uint8_t      type;
     __uint8_t      size;
     __uint16_t     handle;
};

static char const *__bios_vendor   = NULL;
static char const *__bios_version  = NULL;
static char const *__bios_release  = NULL;

static char const *__board_vendor  = NULL;
static char const *__board_type    = NULL;
static char const *__board_version = NULL;
static char const *__board_serial  = NULL;

static int __dmi_probed = 0;

char const * const __dmi_string(struct header *dmi, __uint8_t s);
int __dmi_probe(void);

#define EXPORT(x)   char const * const x(void) { \
                    if(!__dmi_probed) \
                         __dmi_probe(); \
                    return __##x; \
               }

EXPORT(bios_vendor)
EXPORT(bios_version)
EXPORT(bios_release)

EXPORT(board_vendor)
EXPORT(board_type)
EXPORT(board_version)
EXPORT(board_serial)


char const * const __dmi_string(struct header *dmi, __uint8_t s)
{
     __uint8_t *bp=(__uint8_t *)dmi;
     if (!s) return NULL;
     
     bp += dmi->size;

     while(--s)
          bp += strlen((char *)bp) + 1;

     return (char const * const)strip((char *)bp);
}

int __dmi_probe(void)
{
     __uint8_t buf[16];
     off_t size=0;
     int fd_mem;
     
     /* ... at least we tried ... */
     __dmi_probed = 1;

     /* open the rtc device */
     nvprintf(LOG_DEBUG, "Opening %s in O_RDONLY mode...\n", MEM_DEV);
     fd_mem = open(MEM_DEV, O_RDONLY);

     if (fd_mem < 0) {
          nvprintf(LOG_ERR, "%s: %m\n", MEM_DEV);
          return 0;
     }
     
     if (lseek(fd_mem, 0xE0000L, 0) < 0) {
          nvprintf(LOG_ERR, "%s: %m\n", MEM_DEV);
          return 0;
     }
     
     while (size < 0x1FFFF) {
          size += 16;
          if (read(fd_mem, buf, 16) != 16) {
               nvprintf(LOG_ERR, "%s: %m\n", MEM_DEV);
               close(fd_mem);
               return 0;
          }
          
          if (memcmp(buf, "_DMI_", 5)==0) {
               __uint16_t      count = buf[13]<<8  | buf[12];
               __uint16_t      size  = buf[7] <<8  | buf[6];
               __uint32_t      base  = buf[11]<<24 | buf[10]<<16 | buf[9]<<8 | buf[8];
               char           *entry = alloca(size);
               struct header  *dm;
               __uint8_t      *data;
               int             i     = 0;
               
               nvprintf(LOG_DEBUG, "_DMI_ table found: base: 0x%X, size: 0x%X, count: %d\n", base, size, count);
              
               if (lseek(fd_mem, (long)base, SEEK_SET) < 0) {
                    nvprintf(LOG_ERR, "%s: %m\n", MEM_DEV);
                    close(fd_mem);
                    return 0;
               }
               if (read(fd_mem, entry, size) != size) {
                    nvprintf(LOG_ERR, "%s: %m\n", MEM_DEV);
                    close(fd_mem);
                    return 0;
               }
               data = (__uint8_t *)entry;
               
               while(i++<count) {
                    dm = (struct header *)data;
                    if(dm->type == 0) { /* BIOS info */
                         __bios_vendor   =__dmi_string(dm, data[4]);
                         __bios_version  =__dmi_string(dm, data[5]);
                         __bios_release  =__dmi_string(dm, data[8]);
                    } else if(dm->type == 2) { /* Mainboard info */
                         __board_vendor  =__dmi_string(dm, data[4]);
                         __board_type    =__dmi_string(dm, data[5]);
                         __board_version =__dmi_string(dm, data[6]);
                         __board_serial  =__dmi_string(dm, data[7]);
                    } 
                    data += dm->size;
                    while(*data || data[1])
                         data++;
                    data+=2;
               }
          }
     }
     nvprintf(LOG_DEBUG, "Following DMI entries found:\n");
     nvprintf(LOG_DEBUG, " - Mainboard vendor:   %s\n", board_vendor()  );
     nvprintf(LOG_DEBUG, " - Mainboard type:     %s\n", board_type()    );
     nvprintf(LOG_DEBUG, " - Mainboard revision: %s\n", board_version() );
     nvprintf(LOG_DEBUG, " - BIOS vendor:        %s\n", bios_vendor()   );
     nvprintf(LOG_DEBUG, " - BIOS version:       %s\n", bios_version()  );
     nvprintf(LOG_DEBUG, " - BIOS release:       %s\n", bios_release()  );
     close(fd_mem);
     return 1;
}
